<form xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://genshi.edgewall.org/"
    id="${id}"
    name="${name}"
    action="${action}"
    method="${method}"
    class="${css_class}"
    py:attrs="attrs"
>
    <div>
    <py:for each="field in hidden_fields">
        ${Markup(field.display(value_for(field), **args_for(field)))}
    </py:for>
    <py:for each="field in fields">
        ${Markup(field.display(value_for(field), **args_for(field)))} <br />
    </py:for>
    </div>
</form>
