from datetime import date
__all__ = [
    '__VERSION__',
    '__COPYRIGHT__',
    '__AUTHOR__',
    '__LICENSE__',
    '__PACKAGE_NAME__',
    ]

__PACKAGE_NAME__ = "tw.forms"
__VERSION__ = "0.9.7.2"
__AUTHOR__ = "Alberto Valverde Gonzalez"
__EMAIL__ = "alberto@toscat.net"
__COPYRIGHT__ = "2006-%d Alberto Valverde Gonzalez and contributors" % date.today().year
__LICENSE__ = "MIT"
