<textarea xmlns="http://www.w3.org/1999/xhtml" 
          xmlns:py="http://genshi.edgewall.org/"
          id="${id}"
          name="${name}"
          class="${css_class}"
          py:attrs="attrs"
          py:content="value" />
