<div xmlns="http://www.w3.org/1999/xhtml" 
     xmlns:py="http://genshi.edgewall.org/">
    <input type="text" id="${id}" class="${css_class}" name="${name}" 
           value="${strdate}" py:attrs="attrs"/>
    <input type="button" id="${id}_trigger" class="date_field_button" 
           value="${button_text}" />
</div>
