<input xmlns="http://www.w3.org/1999/xhtml"
       xmlns:py="http://genshi.edgewall.org/"
       type="${type}" name="${name}" class="${css_class}" id="${id}"
       value="${value}"
       py:attrs="attrs" />
