<div xmlns="http://www.w3.org/1999/xhtml" 
     xmlns:py="http://genshi.edgewall.org/"
     class="${css_class}"
     py:attrs="attrs">
${value}
<input type="hidden" name="${name}" id="${id}" value="${value}"/>
</div>
